//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of CPCtelera: An Amstrad CPC Game Engine
//  Copyright (C) 2015 ronaldo / Fremos / Cheesetea / ByteRealms (@FranGallegoBR)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//------------------------------------------------------------------------------

//Includes
#include <cpctelera.h>
#include <stdio.h>
#include <string.h>
#include "main.h"
#include "intro.h"
#include "juego.h"
#include "graficos/Sprites_rgas.h"

const u8 disparo_sprite[4] = {
	0x06 , 0x0F , 0x0F , 0x06   // estrella de tamaño 4x4 pixels
};

const TObjeto NaveValoresIniciales = {
	36, 146, 102, &G_NaveEscuelaV2[0], &G_NaveEscuelaH2[0], 36, 16, 6, 8, 0, 0, 128, 0
};


//SFX, no me funciona de la manera que debería con cpctelera 1.4.2 o no lo he conseguido. Lo hago de manera más manual
//Archivo m_sfx.aks con sólo instrumentos (sin música), luego desde el propio Arkos exportar como binario indicando la dirección de memoria donde pondremos 
//lo generado (apuntarlo). Usar cpct_bin2c para convertir el bin generado a array C. Copio abajo el C resultante y hay que ponerle __at con la dirección 
//que habíamos indicado al exportar
__at(0x0040) const unsigned char m_sfx[208] = {
 0x41, 0x54, 0x31, 0x30, 0x01, 0x40, 0x42, 0x0f, 0x02, 0xff, 0xae, 0x00, 0x54, 0x00, 0x5d, 0x00,
 0x70, 0x00, 0xb5, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0d, 0x56, 0x00, 0x01, 0x00, 0x36,
 0x23, 0x36, 0x29, 0x36, 0x2d, 0x36, 0x36, 0x36, 0x2f, 0x36, 0x2a, 0x36, 0x23, 0x0d, 0x56, 0x00,
 0x01, 0x00, 0x3e, 0x24, 0x3e, 0x26, 0x3e, 0x28, 0x3e, 0x26, 0x3e, 0x2a, 0x3e, 0x2d, 0x3e, 0x30,
 0x3e, 0x35, 0x3e, 0x37, 0x3e, 0x38, 0x3e, 0x3c, 0x3e, 0x3e, 0x3e, 0x3f, 0x3e, 0x3f, 0x3e, 0x3e,
 0x3e, 0x3d, 0x3e, 0x3b, 0x3e, 0x39, 0x3e, 0x3a, 0x3e, 0x3b, 0x3a, 0x3c, 0x36, 0x3f, 0x32, 0x3e,
 0x32, 0x3c, 0x2e, 0x3a, 0x2a, 0x38, 0x26, 0x36, 0x22, 0x34, 0x1e, 0x30, 0x16, 0x2c, 0x0e, 0x28,
 0x0a, 0x26, 0x0d, 0x56, 0x00, 0x01, 0x00, 0x36, 0x30, 0x36, 0x30, 0x36, 0x30, 0x36, 0x30, 0x36,
 0x30, 0x36, 0x30, 0x36, 0x30, 0x36, 0x30, 0x36, 0x30, 0x36, 0x30, 0x36, 0x30, 0x36, 0x30, 0x36,
 0x30, 0x36, 0x30, 0x36, 0x30, 0x36, 0x30, 0x02, 0x20, 0x02, 0x20, 0x02, 0x20, 0x02, 0x20, 0x02,
 0x20, 0x02, 0x20, 0x02, 0x20, 0x02, 0x20, 0x02, 0x20, 0x02, 0x20, 0x02, 0x20, 0x02, 0x20, 0x02,
 0x20, 0x02, 0x20, 0x02, 0x20, 0x02, 0x20, 0x0d, 0x56, 0x00, 0x40, 0x00, 0x00, 0x00, 0x0a, 0x01,
 0x00, 0x0b, 0x01, 0x0b, 0x01, 0x0b, 0x01, 0x01, 0x00, 0x01, 0x00, 0x42, 0x80, 0x00, 0x00, 0x00
};


u8 vidas; //Vidas del jugador
u8 nivel; //Nivel por el que va el juego
u16 puntuacion;	//Puntuación de juego
u8 total_elementos; //total de elementos que se usarán de la matriz anterior por cada fase. No puede ser mayor a MAX_ELEMENTOS
u8 vista; // HORIZONTAL o VERTICAL
//u8 idioma; //Guardar el idioma que selecciona el usuario
TObjeto laser; //Disparo de la nave principal
u8 enemigos; //Nº de enemigos por fase para saber que se eliminaron todos
u8 AREA_JUEGO_SUPERIOR_VAR; //El área superior de juego varía en un momento del juego. Empieza en 42, al final de la consola
TObjeto ParedA; //primera Pared para Nivel 3
TObjeto ParedB; //segunda Pared para Nivel 3
u16 timeline; //Cuenta atrás para nivel 3, también servirá para nivel 6
u32 speedrun; //Contador de tiempo jugado en la partida

TObjeto objetos[MAX_ELEMENTOS] ; //Matriz para objetos 3D. La nave será el primer elemento
TDisparo disparos[MAX_ELEMENTOS] ;//Matriz para disparos de los objetos en pantalla

//TScoreboard scoreboard[8]; //Tabla de puntuaciones. No se declara aquí porque tendrá asignación directa (por ensamblador) y ya ya está definido en main.h como extern

u8 paredIzq[(AREA_JUEGO_INFERIOR - LIMITE_NIVEL3)  * 2]; //El límite superior es donde termina la consola, y=42
u8 paredDer[(AREA_JUEGO_INFERIOR - LIMITE_NIVEL3)  * 2];
u8 paredArr1[ANCHO_PANTALLA]; //drawSprite tiene un límite de 64 Bytes, por lo que hay que hacerlo en 2 partes
u8 paredArr2[ANCHO_PANTALLA];
u8 paredAba1[ANCHO_PANTALLA];
u8 paredAba2[ANCHO_PANTALLA];

u8 cheat; //Vidas infinitas

//Aprovechar bytes que no se usan en la memoria de vídeo para meter variables globales
__at(0xC7D0) u8 explosion_FX; //booleano para saber cuándo está sonando la explosión para no cortarlo
__at(0xC7D1) u8 speedrun_digitos[4]; //pasar tiempo jugado a dígitos
__at(0xC7D5) u8 contador_musica; //contador para saber cada cuantas interrupciones hay que mandar bytes de música al chip AY
__at(0xC7D6) u8 vista_pulsada; //booleano para no cambiar de vista constantemente si se mantiene pulsada S
__at(0xC7D7) u8 musica; //booleano 0=música; 1=no 
__at(0xC7D8) f32 factor_vista; //Coeficiente de conversión al cambiar vista
__at(0xCFD0) i8 estrategias[MAX_ESTRATEGIAS]; //Guardar evolución de Aprendizaje de las Estrategias usadas "IA"
__at(0xFFD0) u32 debug_value;
__at(0xFFD4) u8 debug_key;


void dummy_inicializarScoreboard() __naked{ //Inicializar scoreboard con puntos y nombres directamente para ahorrar código,  >100Bytes
	__asm
	_scoreboard::
		.dw 250
		.db 32,18,16,16,18,23
		.dw 200
		.db 11,14,10,23,45,45
		.dw 175
		.db 22,10,35,14,27,45
		.dw 150
		.db 24,27,28,24,23,45
		.dw 100
		.db 28,12,24,29,29,45
		.dw 75
		.db 12,10,27,13,45,45
		.dw 50
		.db 25,14,29,27,10,45
		.dw 10
		.db 21,24,12,20,14,45
	__endasm;
}

void inicializar() {
	cpct_disableFirmware();
	cpct_setBorder(HW_BLACK); 
	cpct_setBlendMode(CPCT_BLEND_OR); //Es el modo que se va a usar casi siempre. Se inicializa a este modo y si se usa otro se vuelve a éste tras usarse cpct_drawSpriteBlended()

	//Las variables globales no pueden ser inicializadas directamente
	//en su declaración con el compilador de cpctelera SDCC, quedarían igualmente a 0, no tomarían ese valor (a no ser que sean constantes)	
	AREA_JUEGO_SUPERIOR_VAR = 42;
}


void main(void) {
  inicializar();
  
  //idioma = ENG;
  //factor_vista = (float) (AREA_JUEGO_INFERIOR - AREA_JUEGO_SUPERIOR_INI) / ANCHO_PANTALLA;  //cast a float, si no se queda a 1 (sin decimales)

  //menuIdiomas();
  while (1) { 	
	introHistoria();
	if (!nivel) 
		tutorial();

	juego();
	cpct_setBorder(HW_BLACK); //A veces se quedaba en gris si muere justo al terminar invulnerabilidad
	hiscores();
	}
	
}
void debug_buddy(u32 value, u8 key) {
	debug_value = value;
	debug_key = key;
} //En .map se puede ver dónde está en memoria para poner punto de interrupción, y en #FFD0 guardará el valor que se le pasa

